(function (Drupal, once, drupalSettings) {
  'use strict';

  Drupal.behaviors.disclosure_menu = {
    attach: function (context, settings) {
      const rootMenuSelector = 'ul.menu.root-menu ';
      const submenuToggleSelector = 'button.menu-toggle';
      const menuLinkSelector = 'li.menu-item > a[data-submenu-id]';
      // @todo make delays configurable.
      const showIntentDelay = 150;
      const hideIntentDelay = 250;

      // Keyboard Navigation.
      once(
        'toggle-init',
        rootMenuSelector + submenuToggleSelector,
        context
      ).forEach((toggle) => {
        const submenuID = toggle.getAttribute('aria-controls');
        const submenu = document.getElementById(submenuID);
        // Hide button if there's no corresponding submenu.
        if (!submenu) {
          toggle.style.display = 'none';
        } else {
          const toggleSubmenu = () => {
            console.log('toggle');
            // Set whether the toggle is expanded.
            if (toggle.getAttribute('aria-expanded') === 'false') {
              toggle.setAttribute('aria-expanded', 'true');
            } else {
              toggle.setAttribute('aria-expanded', 'false');
            }
            // Show the submenu.
            submenu.classList.toggle('hide');
          };

          const hideSubmenu = (event) => {
            if (!submenu.contains(event.relatedTarget)) {
              // Set the toggle to not expanded.
              toggle.setAttribute('aria-expanded', 'false');
              // Hide the submenu
              submenu.classList.add('hide');
            }
          };

          // On toggle button click, toggle the submenu.
          toggle.addEventListener('click', toggleSubmenu);

          // Close submenu after tabbing past it.
          submenu.addEventListener('focusout', hideSubmenu);
          toggle.addEventListener('blur', hideSubmenu);
        }
      });

      // Hover Navigation.
      once('hover-init', rootMenuSelector + menuLinkSelector, context).forEach(
        (link) => {
          const submenuID = link.getAttribute('data-submenu-id');
          const submenu = document.getElementById(submenuID);
          if (!submenu) return;

          const startHideIntent = () => {
            const hideIntent = setTimeout(() => {
              submenu.classList.add('hide');
            }, hideIntentDelay);
            submenu.setAttribute('data-hide-intent', hideIntent);
          };

          const endHideIntent = () => {
            const hideIntent = submenu.getAttribute('data-hide-intent');
            if (hideIntent) {
              clearTimeout(hideIntent);
              submenu.removeAttribute('data-hide-intent');
            }
          };

          const startShowIntent = () => {
            const showIntent = setTimeout(() => {
              submenu.classList.remove('hide');
            }, showIntentDelay);
            submenu.setAttribute('data-show-intent', showIntent);
          };

          const endShowIntent = () => {
            const showIntent = submenu.getAttribute('data-show-intent');
            if (showIntent) {
              clearTimeout(showIntent);
              submenu.removeAttribute('data-show-intent');
            }
          };

          link.addEventListener('pointerover', () => {
            endHideIntent();
            startShowIntent();
          });

          link.addEventListener('pointerout', () => {
            endShowIntent();
            startHideIntent();
          });

          submenu.addEventListener('pointerover', endHideIntent);
          submenu.addEventListener('pointerout', startHideIntent);
        }
      );
    },
    detach: function (context, settings) {},
  };
})(Drupal, once, drupalSettings);
