<?php

namespace Drupal\disclosure_menu\Plugin\Block;

use Drupal\system\Plugin\Block\SystemMenuBlock;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Menu\MenuActiveTrailInterface;
use Drupal\Core\Menu\MenuLinkTreeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a "Disclosure Menu" block.
 *
 * @Block(
 *   id = "disclosure_menu",
 *   admin_label = @Translation("Disclosure menu"),
 *   cache = -1,
 *   category = @Translation("Disclosure Menu"),
 *   deriver = "Drupal\system\Plugin\Derivative\SystemMenuBlock"
 * )
 */
class DisclosureMenuBlock extends SystemMenuBlock {

  /**
   * The active menu trail service.
   *
   * @var \Drupal\Core\Menu\MenuActiveTrailInterface
   */
  protected $menuActiveTrail;

  /**
   * Constructs a new SuperfishBlock.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Menu\MenuLinkTreeInterface $menu_tree
   *   The menu tree service.
   * @param \Drupal\Core\Menu\MenuActiveTrailInterface $menu_active_trail
   *   The active menu trail service.
   */
  public function __construct(array $configuration, $plugin_id, array $plugin_definition, MenuLinkTreeInterface $menu_tree, MenuActiveTrailInterface $menu_active_trail) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $menu_tree, $menu_active_trail);
    $this->menuActiveTrail = $menu_active_trail;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('menu.link_tree'),
      $container->get('menu.active_trail')
    );
  }

  /**
   * Overrides \Drupal\block\BlockBase::defaultConfiguration().
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'submenu_dropdown_levels' => 1,
      'include_default_js' => 1,
      'include_default_css' => 1,
    ];
  }

  /**
   * Overrides \Drupal\block\BlockBase::blockForm().
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    $form['submenu_dropdown_levels'] = [
      '#type' => 'radios',
      '#title' => $this->t('Submenu dropdown levels'),
      '#default_value' => $this->configuration['submenu_dropdown_levels'],
      '#options' => [
        0 => $this->t('None'),
        1 => $this->t('Menu level one submenus'),
        2 => $this->t('Menu level one and two submenus'),
      ],
    ];

    $form['include_default_js'] = [
      '#type' => 'radios',
      '#title' => $this->t('Include default Javascript'),
      '#default_value' => $this->configuration['include_default_js'],
      '#options' => [
        1 => 'Yes',
        0 => 'No',
      ],
    ];

    $form['include_default_css'] = [
      '#type' => 'radios',
      '#title' => $this->t('Include default styles'),
      '#default_value' => $this->configuration['include_default_css'],
      '#options' => [
        1 => 'Yes',
        0 => 'No',
      ],
    ];

    return $form;
  }

  /**
   * Overrides \Drupal\block\BlockBase::blockSubmit().
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $this->configuration['submenu_dropdown_levels'] = $form_state->getValue('submenu_dropdown_levels');
    $this->configuration['include_default_js'] = $form_state->getValue('include_default_js');
    $this->configuration['include_default_css'] = $form_state->getValue('include_default_css');
  }

  /**
   * Implements \Drupal\block\BlockBase::build().
   */
  public function build() {
    $build = parent::build();
    $build['#theme'] = 'menu__disclosure';

    if ($this->configuration['include_default_js']) {
      $build['#attached']['library'][] = 'disclosure_menu/menu';
    }
    if ($this->configuration['include_default_css']) {
      $build['#attached']['library'][] = 'disclosure_menu/styles';
    }

    $build['#submenu_dropdown_levels'] = $this->configuration['submenu_dropdown_levels'];

    return $build;
  }

}
